"""BlinkStick Dataframe Formats

Honestly, this is a hot mess.
"""

from typing import List, Tuple

from ..statevector import StateVector, StateField


class ReportField(StateField):
    """An 8-bit feature report value."""


class ChannelField(StateField):
    """An 8-bit channel value: 0: red, 1: green, 2: blue."""


class IndexField(StateField):
    """An 8-bit index value to specify individual LEDs."""


class Color8Field(StateField):
    """An 8-bit color value."""


class Color24Field(StateField):
    """A 24-bit color value. Accepts an integer or 3-tuple of ints."""

    def __set__(self, obj, value):
        if isinstance(value, tuple):
            value = int.from_bytes(bytes(value), "big")
        super().__set__(obj, value)


class BlinkStickDataframe(StateVector):
    def color(self, color: Tuple[int, int, int]) -> None:
        """Set all LED color values in the dataframe to `color`.

        :param color: Tuple[int, int, int]
        """
        for led in range(0, self.nleds):
            setattr(self, "led{}".format(led), color)

    @property
    def nleds(self) -> int:
        """The number of LEDS in this dataframe."""
        try:
            return self._nleds
        except AttributeError:
            pass
        self._nleds: int = int((len(self) - 16) // 24)
        return self._nleds

    def leds(self) -> List[int]:
        """Return a list of all LED color integer values."""
        values = []
        for led in range(0, self.nleds):
            values.append(getattr(self, f"led{led}"))
        return values


class BlinkStickDataframe8(BlinkStickDataframe):
    def __init__(self):
        super().__init__(6 << 200, 208)

    report = ReportField(200, 8)
    channel = ChannelField(192, 8)
    led0 = Color24Field(168, 24)
    led1 = Color24Field(144, 24)
    led2 = Color24Field(120, 24)
    led3 = Color24Field(96, 24)
    led4 = Color24Field(72, 24)
    led5 = Color24Field(48, 24)
    led6 = Color24Field(24, 24)
    led7 = Color24Field(0, 24)


class BlinkStickDataframe16(BlinkStickDataframe):
    def __init__(self):
        super().__init__(7 << 392, 400)

    report = ReportField(392, 8)
    channel = ChannelField(384, 8)
    led0 = Color24Field(360, 24)
    led1 = Color24Field(336, 24)
    led2 = Color24Field(312, 24)
    led3 = Color24Field(288, 24)
    led4 = Color24Field(264, 24)
    led5 = Color24Field(240, 24)
    led6 = Color24Field(216, 24)
    led7 = Color24Field(192, 24)
    led8 = Color24Field(168, 24)
    led9 = Color24Field(144, 24)
    led10 = Color24Field(120, 24)
    led11 = Color24Field(96, 24)
    led12 = Color24Field(72, 24)
    led13 = Color24Field(48, 24)
    led14 = Color24Field(24, 24)
    led15 = Color24Field(0, 24)


class BlinkStickDataframe32(BlinkStickDataframe):
    def __init__(self):
        super().__init__(8 << 776, 784)

    report = ReportField(776, 8)
    channel = ChannelField(768, 8)
    led0 = Color24Field(744, 24)
    led1 = Color24Field(720, 24)
    led2 = Color24Field(696, 24)
    led3 = Color24Field(672, 24)
    led4 = Color24Field(648, 24)
    led5 = Color24Field(624, 24)
    led6 = Color24Field(600, 24)
    led7 = Color24Field(576, 24)
    led8 = Color24Field(552, 24)
    led9 = Color24Field(528, 24)
    led10 = Color24Field(504, 24)
    led11 = Color24Field(480, 24)
    led12 = Color24Field(456, 24)
    led13 = Color24Field(432, 24)
    led14 = Color24Field(408, 24)
    led15 = Color24Field(384, 24)
    led16 = Color24Field(360, 24)
    led17 = Color24Field(336, 24)
    led18 = Color24Field(312, 24)
    led19 = Color24Field(288, 24)
    led20 = Color24Field(264, 24)
    led21 = Color24Field(240, 24)
    led22 = Color24Field(216, 24)
    led23 = Color24Field(192, 24)
    led24 = Color24Field(168, 24)
    led25 = Color24Field(144, 24)
    led26 = Color24Field(120, 24)
    led27 = Color24Field(96, 24)
    led28 = Color24Field(72, 24)
    led29 = Color24Field(48, 24)
    led30 = Color24Field(24, 24)
    led31 = Color24Field(0, 24)


class BlinkStickDataframe64(BlinkStickDataframe):
    def __init__(self):
        super().__init__(9 << 1544, 1552)

    report = ReportField(1544, 8)
    channel = ChannelField(1536, 8)
    led0 = Color24Field(1512, 24)
    led1 = Color24Field(1488, 24)
    led2 = Color24Field(1464, 24)
    led3 = Color24Field(1440, 24)
    led4 = Color24Field(1416, 24)
    led5 = Color24Field(1392, 24)
    led6 = Color24Field(1368, 24)
    led7 = Color24Field(1344, 24)
    led8 = Color24Field(1320, 24)
    led9 = Color24Field(1296, 24)
    led10 = Color24Field(1272, 24)
    led11 = Color24Field(1248, 24)
    led12 = Color24Field(1224, 24)
    led13 = Color24Field(1200, 24)
    led14 = Color24Field(1176, 24)
    led15 = Color24Field(1152, 24)
    led16 = Color24Field(1128, 24)
    led17 = Color24Field(1104, 24)
    led18 = Color24Field(1080, 24)
    led19 = Color24Field(1056, 24)
    led20 = Color24Field(1032, 24)
    led21 = Color24Field(1008, 24)
    led22 = Color24Field(984, 24)
    led23 = Color24Field(960, 24)
    led24 = Color24Field(936, 24)
    led25 = Color24Field(912, 24)
    led26 = Color24Field(888, 24)
    led27 = Color24Field(864, 24)
    led28 = Color24Field(840, 24)
    led29 = Color24Field(816, 24)
    led30 = Color24Field(792, 24)
    led31 = Color24Field(768, 24)
    led32 = Color24Field(744, 24)
    led33 = Color24Field(720, 24)
    led34 = Color24Field(696, 24)
    led35 = Color24Field(672, 24)
    led36 = Color24Field(648, 24)
    led37 = Color24Field(624, 24)
    led38 = Color24Field(600, 24)
    led39 = Color24Field(576, 24)
    led40 = Color24Field(552, 24)
    led41 = Color24Field(528, 24)
    led42 = Color24Field(504, 24)
    led43 = Color24Field(480, 24)
    led44 = Color24Field(456, 24)
    led45 = Color24Field(432, 24)
    led46 = Color24Field(408, 24)
    led47 = Color24Field(384, 24)
    led48 = Color24Field(360, 24)
    led49 = Color24Field(336, 24)
    led50 = Color24Field(312, 24)
    led51 = Color24Field(288, 24)
    led52 = Color24Field(264, 24)
    led53 = Color24Field(240, 24)
    led54 = Color24Field(216, 24)
    led55 = Color24Field(192, 24)
    led56 = Color24Field(168, 24)
    led57 = Color24Field(144, 24)
    led58 = Color24Field(120, 24)
    led59 = Color24Field(96, 24)
    led60 = Color24Field(72, 24)
    led61 = Color24Field(48, 24)
    led62 = Color24Field(24, 24)
    led63 = Color24Field(0, 24)


class BlinkStickDataframe192(StateVector):
    def __init__(self):
        super().__init(29 << 4616, 4624)

    report = ReportField(4616, 8)
    channel = ChannelField(4608, 8)
    led0 = Color24Field(4584, 24)
    led1 = Color24Field(4560, 24)
    led2 = Color24Field(4536, 24)
    led3 = Color24Field(4512, 24)
    led4 = Color24Field(4488, 24)
    led5 = Color24Field(4464, 24)
    led6 = Color24Field(4440, 24)
    led7 = Color24Field(4416, 24)
    led8 = Color24Field(4392, 24)
    led9 = Color24Field(4368, 24)
    led10 = Color24Field(4344, 24)
    led11 = Color24Field(4320, 24)
    led12 = Color24Field(4296, 24)
    led13 = Color24Field(4272, 24)
    led14 = Color24Field(4248, 24)
    led15 = Color24Field(4224, 24)
    led16 = Color24Field(4200, 24)
    led17 = Color24Field(4176, 24)
    led18 = Color24Field(4152, 24)
    led19 = Color24Field(4128, 24)
    led20 = Color24Field(4104, 24)
    led21 = Color24Field(4080, 24)
    led22 = Color24Field(4056, 24)
    led23 = Color24Field(4032, 24)
    led24 = Color24Field(4008, 24)
    led25 = Color24Field(3984, 24)
    led26 = Color24Field(3960, 24)
    led27 = Color24Field(3936, 24)
    led28 = Color24Field(3912, 24)
    led29 = Color24Field(3888, 24)
    led30 = Color24Field(3864, 24)
    led31 = Color24Field(3840, 24)
    led32 = Color24Field(3816, 24)
    led33 = Color24Field(3792, 24)
    led34 = Color24Field(3768, 24)
    led35 = Color24Field(3744, 24)
    led36 = Color24Field(3720, 24)
    led37 = Color24Field(3696, 24)
    led38 = Color24Field(3672, 24)
    led39 = Color24Field(3648, 24)
    led40 = Color24Field(3624, 24)
    led41 = Color24Field(3600, 24)
    led42 = Color24Field(3576, 24)
    led43 = Color24Field(3552, 24)
    led44 = Color24Field(3528, 24)
    led45 = Color24Field(3504, 24)
    led46 = Color24Field(3480, 24)
    led47 = Color24Field(3456, 24)
    led48 = Color24Field(3432, 24)
    led49 = Color24Field(3408, 24)
    led50 = Color24Field(3384, 24)
    led51 = Color24Field(3360, 24)
    led52 = Color24Field(3336, 24)
    led53 = Color24Field(3312, 24)
    led54 = Color24Field(3288, 24)
    led55 = Color24Field(3264, 24)
    led56 = Color24Field(3240, 24)
    led57 = Color24Field(3216, 24)
    led58 = Color24Field(3192, 24)
    led59 = Color24Field(3168, 24)
    led60 = Color24Field(3144, 24)
    led61 = Color24Field(3120, 24)
    led62 = Color24Field(3096, 24)
    led63 = Color24Field(3072, 24)
    led64 = Color24Field(3048, 24)
    led65 = Color24Field(3024, 24)
    led66 = Color24Field(3000, 24)
    led67 = Color24Field(2976, 24)
    led68 = Color24Field(2952, 24)
    led69 = Color24Field(2928, 24)
    led70 = Color24Field(2904, 24)
    led71 = Color24Field(2880, 24)
    led72 = Color24Field(2856, 24)
    led73 = Color24Field(2832, 24)
    led74 = Color24Field(2808, 24)
    led75 = Color24Field(2784, 24)
    led76 = Color24Field(2760, 24)
    led77 = Color24Field(2736, 24)
    led78 = Color24Field(2712, 24)
    led79 = Color24Field(2688, 24)
    led80 = Color24Field(2664, 24)
    led81 = Color24Field(2640, 24)
    led82 = Color24Field(2616, 24)
    led83 = Color24Field(2592, 24)
    led84 = Color24Field(2568, 24)
    led85 = Color24Field(2544, 24)
    led86 = Color24Field(2520, 24)
    led87 = Color24Field(2496, 24)
    led88 = Color24Field(2472, 24)
    led89 = Color24Field(2448, 24)
    led90 = Color24Field(2424, 24)
    led91 = Color24Field(2400, 24)
    led92 = Color24Field(2376, 24)
    led93 = Color24Field(2352, 24)
    led94 = Color24Field(2328, 24)
    led95 = Color24Field(2304, 24)
    led96 = Color24Field(2280, 24)
    led97 = Color24Field(2256, 24)
    led98 = Color24Field(2232, 24)
    led99 = Color24Field(2208, 24)
    led100 = Color24Field(2184, 24)
    led101 = Color24Field(2160, 24)
    led102 = Color24Field(2136, 24)
    led103 = Color24Field(2112, 24)
    led104 = Color24Field(2088, 24)
    led105 = Color24Field(2064, 24)
    led106 = Color24Field(2040, 24)
    led107 = Color24Field(2016, 24)
    led108 = Color24Field(1992, 24)
    led109 = Color24Field(1968, 24)
    led110 = Color24Field(1944, 24)
    led111 = Color24Field(1920, 24)
    led112 = Color24Field(1896, 24)
    led113 = Color24Field(1872, 24)
    led114 = Color24Field(1848, 24)
    led115 = Color24Field(1824, 24)
    led116 = Color24Field(1800, 24)
    led117 = Color24Field(1776, 24)
    led118 = Color24Field(1752, 24)
    led119 = Color24Field(1728, 24)
    led120 = Color24Field(1704, 24)
    led121 = Color24Field(1680, 24)
    led122 = Color24Field(1656, 24)
    led123 = Color24Field(1632, 24)
    led124 = Color24Field(1608, 24)
    led125 = Color24Field(1584, 24)
    led126 = Color24Field(1560, 24)
    led127 = Color24Field(1536, 24)
    led128 = Color24Field(1512, 24)
    led129 = Color24Field(1488, 24)
    led130 = Color24Field(1464, 24)
    led131 = Color24Field(1440, 24)
    led132 = Color24Field(1416, 24)
    led133 = Color24Field(1392, 24)
    led134 = Color24Field(1368, 24)
    led135 = Color24Field(1344, 24)
    led136 = Color24Field(1320, 24)
    led137 = Color24Field(1296, 24)
    led138 = Color24Field(1272, 24)
    led139 = Color24Field(1248, 24)
    led140 = Color24Field(1224, 24)
    led141 = Color24Field(1200, 24)
    led142 = Color24Field(1176, 24)
    led143 = Color24Field(1152, 24)
    led144 = Color24Field(1128, 24)
    led145 = Color24Field(1104, 24)
    led146 = Color24Field(1080, 24)
    led147 = Color24Field(1056, 24)
    led148 = Color24Field(1032, 24)
    led149 = Color24Field(1008, 24)
    led150 = Color24Field(984, 24)
    led151 = Color24Field(960, 24)
    led152 = Color24Field(936, 24)
    led153 = Color24Field(912, 24)
    led154 = Color24Field(888, 24)
    led155 = Color24Field(864, 24)
    led156 = Color24Field(840, 24)
    led157 = Color24Field(816, 24)
    led158 = Color24Field(792, 24)
    led159 = Color24Field(768, 24)
    led160 = Color24Field(744, 24)
    led161 = Color24Field(720, 24)
    led162 = Color24Field(696, 24)
    led163 = Color24Field(672, 24)
    led164 = Color24Field(648, 24)
    led165 = Color24Field(624, 24)
    led166 = Color24Field(600, 24)
    led167 = Color24Field(576, 24)
    led168 = Color24Field(552, 24)
    led169 = Color24Field(528, 24)
    led170 = Color24Field(504, 24)
    led171 = Color24Field(480, 24)
    led172 = Color24Field(456, 24)
    led173 = Color24Field(432, 24)
    led174 = Color24Field(408, 24)
    led175 = Color24Field(384, 24)
    led176 = Color24Field(360, 24)
    led177 = Color24Field(336, 24)
    led178 = Color24Field(312, 24)
    led179 = Color24Field(288, 24)
    led180 = Color24Field(264, 24)
    led181 = Color24Field(240, 24)
    led182 = Color24Field(216, 24)
    led183 = Color24Field(192, 24)
    led184 = Color24Field(168, 24)
    led185 = Color24Field(144, 24)
    led186 = Color24Field(120, 24)
    led187 = Color24Field(96, 24)
    led188 = Color24Field(72, 24)
    led189 = Color24Field(48, 24)
    led190 = Color24Field(24, 24)
    led191 = Color24Field(0, 24)
